/*
* Copyright (C) 2021, KylinSoft Co., Ltd.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
*
*/
#include "titlebar.h"
#include "ui_titlebar.h"
#include <QMessageBox>
#include <gsettings.hpp>
#include <kylin_system/user_manual.hpp>

TitleBar::TitleBar(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::TitleBar),
    m_menu(new QMenu(this))
{
    ui->setupUi(this);

    // appname
    QFont appnameFont;
    appnameFont.setPixelSize(TitlebarFontSize);
    ui->m_appnameLabel->setFont(appnameFont);
//    ui->m_appnameLabel->adjustSize();

    m_menu->addAction(tr("Help"), this, [ = ]() {
        showHelpDialog();
    });//, QKeySequence(Qt::Key_F1)

    m_menu->addAction(tr("About"), this, [ = ]() {
        g_user_signal->showAboutDialog();
//        using namespace kdk;
//        m_aboutWindow = new KAboutDialog(this,QIcon::fromTheme("kylin-camera"),tr(appShowingName.toLocal8Bit()),tr("Version: ") + appVersion);
//        m_aboutWindow->setBodyText(tr("Message provides text chat and file transfer functions in the LAN. "
//                          "There is no need to build a server. "
//                          "It supports multiple people to interact at the same time "
//                          "and send and receive in parallel."));
//        m_aboutWindow->setBodyTextVisiable(true);
//        m_aboutWindow->setWindowModality(Qt::WindowModal);
//        m_aboutWindow->setWindowModality(Qt::ApplicationModal);
    });

    m_menu->addAction(tr("Exit"), [ = ]() {
        KyInfo() << "window close.";
        if (g_user_signal->exitWindowWithSaveFlag) {
            int saveFlag = warnCloseWindow();
            if (saveFlag == 1) {
                KyInfo() << "close window save as";
                g_user_signal->closeWindowSaveAs();
            }
        }

        g_user_signal->exitApplication();
    } );
    ui->m_OptionBtn->setMenu(m_menu);
    connect(kdk::kabase::Gsettings::getPoint(), &kdk::kabase::Gsettings::systemFontSizeChange, this,&TitleBar::fontSizeChanged);

}

TitleBar::~TitleBar()
{
    delete ui;
}

void TitleBar::mouseDoubleClickEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) {
        if (window()->isMaximized()) {
            updateMaxButtonStatus(false);
            window()->showNormal();
        } else if (! window()->isFullScreen()) {
            updateMaxButtonStatus(true);
            window()->showMaximized();
        }
    }
    QWidget::mouseDoubleClickEvent(event);
}

void TitleBar::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);
    QPainter p(this);
    p.setRenderHint(QPainter::Antialiasing);
    QPainterPath rectPath;
    rectPath.addRoundedRect(this->rect(), 0, 0);
    QStyleOption opt;
    opt.init(this);

    QColor mainColor;
    mainColor = opt.palette.color(QPalette::Base);
    p.fillPath(rectPath,QBrush(mainColor));
}

void TitleBar::updateMaxButtonStatus(bool isMax)
{
    if (isMax) {
        ui->m_maxBtn->setIcon(QIcon::fromTheme("window-restore-symbolic"));
        ui->m_maxBtn->setToolTip(tr("Restore"));
    } else {
        ui->m_maxBtn->setIcon(QIcon::fromTheme("window-maximize-symbolic"));
        ui->m_maxBtn->setToolTip(tr("Maximize"));
    }
}

void TitleBar::on_m_minBtn_clicked()
{
    g_user_signal->minimumWindowSignal();
}

void TitleBar::on_m_maxBtn_clicked(bool checked)
{
    KyInfo() << "checked: " << checked;

    g_user_signal->maximumWindowSignal();
}

void TitleBar::on_m_closeBtn_clicked()
{
    KyInfo() << "window close.";
    if (g_user_signal->exitWindowWithSaveFlag) {
        int saveFlag = warnCloseWindow();
        if (saveFlag == 1) {
            KyInfo() << "close window save as";
            g_user_signal->closeWindowSaveAs();
        }
    }
    g_user_signal->exitApplication();

}

void TitleBar::showHelpDialog()
{
    kdk::kabase::UserManualManagement userManualTest;
    if (!userManualTest.callUserManual(kdk::kabase::AppName::KylinScanner)) {
        qCritical() << "user manual call fail!";
    }
    return;
//    QString appName = "kylin-scanner";
//    KYCDaemonDbusObject *ipcDbus = new KYCDaemonDbusObject();

//    if (!ipcDbus->daemonIsNotRunning()) {
//        ipcDbus->showGuide(appName);
//    } else {
//        KyInfo() << "Guide is Running.";
//    }
}

int TitleBar::warnCloseWindow()
{

    QMessageBox *msgBox = new QMessageBox();
    msgBox->setModal(Qt::WindowModal);
    msgBox->setAttribute(Qt::WA_ShowModal, true);

    msgBox->setText(tr("The current document is not saved. Do you want to save it?"));
    msgBox->setIcon(QMessageBox::Warning);
//    msgBox->setWindowIcon(QIcon::fromTheme("kylin-scanner"));
    msgBox->setWindowTitle(tr("Scanner"));
//    msgBox->setStandardButtons(QMessageBox::Yes);
    msgBox->setContextMenuPolicy(Qt::NoContextMenu);
//    msgBox->button(QMessageBox::Yes)->setText(tr("Yes"));
    QAbstractButton *noButton = (QAbstractButton *)msgBox->addButton(tr("Straight &Exit"), QMessageBox::NoRole);
//    QAbstractButton *thinkButton = (QAbstractButton *)msgBox->addButton(QString(tr("Save &Not exit")), QMessageBox::NoRole);
    QAbstractButton *yesButton =  (QAbstractButton *)msgBox->addButton(tr("&Save Exit"), QMessageBox::YesRole);

    msgBox->setWindowFlags(Qt::CustomizeWindowHint | Qt::WindowMinimizeButtonHint | Qt::WindowMaximizeButtonHint);

    QWidget *widget = nullptr;
    QWidgetList widgetList = QApplication::allWidgets();
    for (int i=0; i<widgetList.length(); ++i) {
        if (widgetList.at(i)->objectName() == "MainWindow") {
            widget = widgetList.at(i);
        }
    }
    if (widget) {
        msgBox->setParent(widget);
        QRect rect = widget->geometry();
        int x = rect.x() + rect.width()/2 - msgBox->width()/2;
        int y = rect.y() + rect.height()/2 - msgBox->height()/2;
        msgBox->move(x,y);
    }


    msgBox->exec();

#if 0
    QTimer* timer = new QTimer();
    timer->start(3000);
    timer->setSingleShot(true);

    connect(timer, &QTimer::timeout, msgBox, &QMessageBox::hide);
#endif

    if (msgBox->clickedButton() == yesButton) {
        KyInfo() << "Accepeted !";
        return 1;
    } else {
        KyInfo() << "Rejected !";
        return 0;
    }

}

void TitleBar::fontSizeChanged()
{
    int systemFontSize = kdk::kabase::Gsettings::getSystemFontSize().toInt();
    QFont font;
    font.setPointSize(systemFontSize);
    ui->m_appnameLabel->setFont(font);
}
